#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Harfbuzz(Tarball, Meson):
    def __init__(self):
        self.version = "5.3.0"
        Project.__init__(
            self,
            "harfbuzz",
            archive_url=f"https://github.com/harfbuzz/harfbuzz/releases/download/{self.version}/harfbuzz-{self.version}.tar.xz",
            hash="a05e19e3f52da24ed071522f0fddf872157d7d25e869cfd156cd6f1e81c42152",
            dependencies=["meson", "cmake", "python", "freetype", "pkg-config", "glib"],
        )

        if Project.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            self.add_param("-Dintrospection=enabled")
        else:
            self.add_param("-Dintrospection=disabled")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\harfbuzz")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import NullExpander
from gvsbuild.utils.base_project import Project, project_add


@project_add
class CheckLibs(NullExpander, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "check-libs",
            dependencies=[
                # Used to build the various tests
                "meson",
                "ninja",
                "pkg-config",
                # libraries to test, hopefully all the one we build!
                "atk",
                "cairo",
                "freetype",
                "gdk-pixbuf",
                "glib",
                "json-glib",
                "libarchive",
                "libcurl",
                "libffi",
                "libjpeg-turbo",
                "libpng",
                "libtiff-4",
                "libxml2",
                "libyuv",
                "pango",
                "zlib",
            ],
            version="0.1.0",
        )

    def build(self):
        Meson.build(self, make_tests=True)
        self.install(r".\COPYING share\doc\check-libs")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class DcvColorPrimitives(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "dcv-color-primitives",
            archive_url="https://github.com/aws/dcv-color-primitives/archive/v0.5.1.tar.gz",
            archive_file_name="dcv-color-primitives-0.5.1.tar.gz",
            hash="6d3067308e5b419c869e256e413b8f9d1fbda56578336e2190911611e9e7f246",
            dependencies=["ninja", "meson", "pkg-config", "cargo"],
        )

    def build(self):
        Meson.build(self, make_tests=False)
        self.install(r".\LICENSE share\doc\dcv-color-primitives")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import GitRepo
from gvsbuild.utils.base_project import project_add


@project_add
class Emeus(GitRepo, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "emeus",
            repo_url="https://github.com/ebassi/emeus.git",
            fetch_submodules=False,
            tag=None,
            dependencies=["ninja", "meson", "pkg-config", "gtk3"],
            patches=["00_win_no_script.patch"],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "true"
        else:
            enable_gi = "false"

        self.add_param("-Ddocs=false")
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self, make_tests=True)
        self.install(r".\COPYING.txt share\doc\emeus")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Enchant(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "enchant",
            archive_url="https://dl.hexchat.net/gtk-win32/src/enchant-1.6.1.tar.xz",
            hash="d6cddd2621589ca8becaba1bfe8d3668f7d6592743664ef0e1a35543971fbe6e",
            dependencies=["glib"],
        )

    def build(self):
        x64_param = "X64=1" if self.builder.x64 else ""
        self.push_location(r".\src")

        # Exec nmake /nologo -f makefile.mak clean
        self.exec_vs(
            r"nmake /nologo -f makefile.mak DLL=1 "
            + x64_param
            + r" MFLAGS=-MD GLIBDIR=%(gtk_dir)s\include\glib-2.0"
        )

        self.pop_location()

        self.install(
            r".\bin\release\enchant.exe "
            r".\bin\release\pdb\enchant.pdb "
            r".\bin\release\enchant-lsmod.exe "
            r".\bin\release\pdb\enchant-lsmod.pdb "
            r".\bin\release\test-enchant.exe "
            r".\bin\release\pdb\test-enchant.pdb "
            r".\bin\release\libenchant.dll "
            r".\bin\release\pdb\libenchant.pdb "
            r"bin"
        )

        self.install(r".\fonts.conf " r".\fonts.dtd " r"etc\fonts")

        self.install(
            r".\src\enchant.h "
            r".\src\enchant++.h "
            r".\src\enchant-provider.h "
            r"include\enchant"
        )

        self.install(r".\bin\release\libenchant.lib lib")

        self.install(
            r".\bin\release\libenchant_ispell.dll "
            r".\bin\release\libenchant_ispell.lib "
            r".\bin\release\pdb\libenchant_ispell.pdb "
            r".\bin\release\libenchant_myspell.dll "
            r".\bin\release\libenchant_myspell.lib "
            r".\bin\release\pdb\libenchant_myspell.pdb "
            r"lib\enchant"
        )

        self.install(r".\COPYING.LIB share\doc\enchant")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#  Copyright (C) 2017 - Daniele Forghieri
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Default groups of projects."""

from .utils.base_group import Group, group_add
from .utils.base_project import Project, ProjectType


@group_add
class Group_Tools(Group):
    def __init__(self):
        Group.__init__(
            self,
            "tools",
            dependencies=[
                "cargo",
                "cmake",
                "go",
                "meson",
                "msys2",
                "nasm",
                "ninja",
                "nuget",
                "perl",
                "python",
                "yasm",
            ],
        )


@group_add
class Group_Gtk3_Full(Group):
    def __init__(self):
        Group.__init__(
            self,
            "gtk3-full",
            dependencies=[
                "adwaita-icon-theme",
                "clutter",
                "emeus",
                "gtk3",
                "gtksourceview4",
                "hicolor-icon-theme",
                "wing",
            ],
        )


@group_add
class Group_Tools_Check(Group):
    """Group to use all the tools handled by the script, to see at a glance if
    everything seems ok after (big) changes on the tools."""

    def __init__(self):
        Group.__init__(
            self,
            "tools-check",
            dependencies=[
                "lmdb",
                "x264",
                "libjpeg-turbo",
            ],
        )


@group_add
class Group_All(Group):
    def __init__(self):
        all_prj = [x.name for x in Project._projects if x.type == ProjectType.PROJECT]
        Group.__init__(self, "all", dependencies=all_prj)



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Luajit(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "luajit",
            archive_url="http://luajit.org/download/LuaJIT-2.1.0-beta3.tar.gz",
            hash="1ad2e34b111c802f9d0cdf019e986909123237a28c746b21295b63c9e785d9c3",
            patches=["set-paths.patch"],
        )

    def build(self):
        option = "debug" if self.builder.opts.configuration == "debug" else ""
        self.push_location("src")

        self.exec_vs(r".\msvcbuild " + option)

        self.install(
            r".\lua.h .\lualib.h .\luaconf.h .\lauxlib.h .\luajit.h include\luajit-2.1"
        )
        self.install(r".\luajit.exe .\lua51.dll .\lua51.pdb bin")
        self.install(r".\lua51.lib lib")
        self.pop_location()

        self.install(r".\README .\COPYRIGHT share\doc\luajit")
        self.install_pc_files()



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class WinIconv(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "win-iconv",
            archive_url="https://github.com/win-iconv/win-iconv/archive/v0.0.8.tar.gz",
            archive_file_name="win-iconv-0.0.8.tar.gz",
            hash="23adea990a8303c6e69e32a64a30171efcb1b73824a1c2da1bbf576b0ae7c520",
            dependencies=[
                "cmake",
                "ninja",
            ],
        )

    def build(self):
        CmakeProject.build(
            self, use_ninja=True, cmake_params="-DBUILD_TEST=1", make_tests=True
        )

        self.install(r".\COPYING share\doc\win-iconv")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Gettext(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "gettext",
            archive_url="http://ftp.gnu.org/pub/gnu/gettext/gettext-0.21.tar.xz",
            hash="d20fcbb537e02dcf1383197ba05bd0734ef7bf5db06bdb241eb69b7d16b73192",
            dependencies=["python", "win-iconv"],
            patches=[
                "gettext-runtime-c99.patch",
                "gettext-tools-c99.patch",
                "gettext-tools-gnulib-memset.patch",
                "libtextstyle-c99.patch",
            ],
        )

    def build(self):
        self.push_location(r".\nmake")
        self.exec_vs(
            r'nmake /nologo /f Makefile.vc CFG=%(configuration)s PYTHON="%(python_dir)s\python.exe" PREFIX="%(gtk_dir)s"',
            add_path=os.path.join(self.builder.opts.msys_dir, "usr", "bin"),
        )
        self.pop_location()

        self.push_location(
            r".\nmake\vs%s\%s\%s"
            % (
                self.builder.opts.vs_ver,
                self.builder.opts.configuration,
                self.builder.opts.platform,
            )
        )
        self.install(r".\asprintf.dll bin")
        self.install(r".\asprintf.pdb bin")
        self.install(r".\intl.dll bin")
        self.install(r".\intl.pdb bin")
        self.install(r".\envsubst.exe bin")
        self.install(r".\envsubst.pdb bin")
        self.install(r".\gettext.exe bin")
        self.install(r".\gettext.pdb bin")
        self.install(r".\ngettext.exe bin")
        self.install(r".\ngettext.pdb bin")
        self.install(r".\gettextpo.dll bin")
        self.install(r".\gettextpo.pdb bin")
        self.install(r".\gettextlib-*.dll bin")
        self.install(r".\gettextlib-*.pdb bin")
        self.install(r".\gettextsrc-*.dll bin")
        self.install(r".\gettextsrc-*.pdb bin")
        self.install(r".\msg*.exe bin")
        self.install(r".\msg*.pdb bin")
        self.install(r".\xgettext.exe bin")
        self.install(r".\xgettext.pdb bin")
        self.install(r".\recode-sr-latin.exe bin")
        self.install(r".\recode-sr-latin.pdb bin")
        self.install(r".\textstyle.dll bin")
        self.install(r".\textstyle.pdb bin")

        self.install(r".\asprintf.lib lib")
        self.install(r".\intl.lib lib")
        self.install(r".\gettextpo.lib lib")
        self.pop_location()

        self.push_location(r".\msvc")
        self.install(r".\gettext-runtime\libasprintf\autosprintf.h include")
        self.install(r".\gettext-runtime\intl\libgnuintl.h include")
        self.install(r".\gettext-tools\libgettextpo\gettext-po.h include")
        self.pop_location()

        self.install(r".\gettext-tools\its\*.its share\gettext\its")
        self.install(r".\gettext-tools\its\*.loc share\gettext\its")
        self.install(r".\COPYING share\doc\gettext")

    def post_install(self):
        self.builder.exec_msys(
            ["mv", "libgnuintl.h", "libintl.h"],
            working_dir=os.path.join(self.builder.gtk_dir, "include"),
        )



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Protobuf(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "protobuf",
            archive_url="https://github.com/protocolbuffers/protobuf/releases/download/v3.15.8/protobuf-cpp-3.15.8.tar.gz",
            hash="9b57647b898e45253c98fae35146f6a5e9e788817d29019f9280270c951a0038",
            dependencies=[
                "cmake",
                "zlib",
                "ninja",
            ],
        )

    def build(self):
        # We need to compile with STATIC_RUNTIME off since protobuf-c also compiles with it OFF
        CmakeProject.build(
            self,
            cmake_params=r'-Dprotobuf_DEBUG_POSTFIX="" -Dprotobuf_BUILD_TESTS=OFF -Dprotobuf_WITH_ZLIB=ON -Dprotobuf_MSVC_STATIC_RUNTIME=OFF',
            use_ninja=True,
            source_part="cmake",
        )

        self.install(r".\LICENSE share\doc\protobuf")


@project_add
class ProtobufC(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "protobuf-c",
            archive_url="https://github.com/protobuf-c/protobuf-c/releases/download/v1.3.3/protobuf-c-1.3.3.tar.gz",
            hash="22956606ef50c60de1fabc13a78fbc50830a0447d780467d3c519f84ad527e78",
            dependencies=[
                "cmake",
                "protobuf",
                "ninja",
            ],
        )

    def build(self):
        CmakeProject.build(self, use_ninja=True, source_part="build-cmake")

        self.install(r".\LICENSE share\doc\protobuf-c")
        self.install_pc_files()



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Portaudio(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "portaudio",
            archive_url="http://files.portaudio.com/archives/pa_stable_v190700_20210406.tgz",
            hash="47efbf42c77c19a05d22e627d42873e991ec0c1357219c0d74ce6a2948cb2def",
            dependencies=[
                "cmake",
                "ninja",
            ],
        )

    def build(self):
        CmakeProject.build(
            self,
            cmake_params="-DPA_DLL_LINK_WITH_STATIC_RUNTIME=off",
            use_ninja=True,
            out_of_source=False,
        )



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Cogl(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "cogl",
            archive_url="https://download.gnome.org/sources/cogl/1.22/cogl-1.22.8.tar.xz",
            hash="a805b2b019184710ff53d0496f9f0ce6dcca420c141a0f4f6fcc02131581d759",
            dependencies=["python", "glib", "cairo", "pango", "gdk-pixbuf"],
            patches=[
                "001-cogl-missing-symbols.patch",
                "002-cogl-pango-missing-symbols.patch",
            ],
        )

    def build(self):
        self.builder.mod_env("INCLUDE", f"{self.builder.gtk_dir}\\include\\harfbuzz")
        self.exec_msbuild_gen(r"build\win32", "cogl.sln", add_pars="/p:UseEnv=True")

        self.install(r".\COPYING share\doc\cogl")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GStreamer(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gstreamer",
            archive_url="https://gstreamer.freedesktop.org/src/gstreamer/gstreamer-1.20.2.tar.xz",
            hash="df24e8792691a02dfe003b3833a51f1dbc6c3331ae625d143b17da939ceb5e0a",
            dependencies=["meson", "ninja", "glib", "orc"],
        )

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        Meson.build(
            self, add_path=add_path, meson_params="-Dtests=disabled -Dexamples=disabled"
        )
        self.install(r".\COPYING share\doc\gstreamer")


@project_add
class Orc(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "orc",
            archive_url="https://gstreamer.freedesktop.org/src/orc/orc-0.4.32.tar.xz",
            hash="a66e3d8f2b7e65178d786a01ef61f2a0a0b4d0b8370de7ce134ba73da4af18f0",
            dependencies=[
                "ninja",
                "meson",
            ],
        )

    def build(self):
        Meson.build(self, meson_params="-Dbenchmarks=disabled -Dtools=enabled")
        self.install(r"COPYING share\doc\orc")


@project_add
class GstPluginsBase(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-base",
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-base/gst-plugins-base-1.20.2.tar.xz",
            hash="ab0656f2ad4d38292a803e0cb4ca090943a9b43c8063f650b4d3e3606c317f17",
            dependencies=["meson", "ninja", "gtk3", "gstreamer", "opus"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-base")


@project_add
class GstPluginsGood(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-good",
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-good/gst-plugins-good-1.20.2.tar.xz",
            hash="83589007bf002b8f9ef627718f308c16d83351905f0db8e85c3060f304143aae",
            dependencies=[
                "meson",
                "ninja",
                "glib",
                "gstreamer",
                "gst-plugins-base",
                "libvpx",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-good")


@project_add
class GstPluginsBad(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-bad",
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-bad/gst-plugins-bad-1.20.2.tar.xz",
            hash="4adc4c05f41051f8136b80cda99b0d049a34e777832f9fea7c5a70347658745b",
            dependencies=["meson", "ninja", "glib", "gstreamer", "gst-plugins-base"],
            patches=[
                "wasapi-Implement-default-audio-channel-mask.patch",
                "wasapisink-reduce-buffer-latency.patch",
            ],
        )
        self.add_param("-Dcurl=disabled")
        self.add_param("-Dcurl-ssh2=disabled")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-bad")


@project_add
class GstPython(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-python",
            archive_url="https://gstreamer.freedesktop.org/src/gst-python/gst-python-1.20.2.tar.xz",
            hash="853ea35a1088c762fb703e5aea9c30031a19222b59786b6599956e154620fa2f",
            dependencies=["meson", "ninja", "glib", "gstreamer", "pygobject"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-python")



# flake8: noqa

from gvsbuild.projects.adwaita_icon_theme import AdwaitaIconTheme
from gvsbuild.projects.atk import Atk
from gvsbuild.projects.boringssl import BoringSSL
from gvsbuild.projects.cairo import Cairo
from gvsbuild.projects.check_libs import CheckLibs
from gvsbuild.projects.clutter import Clutter
from gvsbuild.projects.cogl import Cogl
from gvsbuild.projects.cyrus_sasl import CyrusSasl
from gvsbuild.projects.dcv_color_primitives import DcvColorPrimitives
from gvsbuild.projects.dev_shell import DevShell
from gvsbuild.projects.emeus import Emeus
from gvsbuild.projects.enchant import Enchant
from gvsbuild.projects.ffmpeg import Ffmpeg
from gvsbuild.projects.fontconfig import Fontconfig
from gvsbuild.projects.freerdp import FreeRDP
from gvsbuild.projects.freetype import Freetype
from gvsbuild.projects.fribidi import Fribidi
from gvsbuild.projects.gdk_pixbuf import GdkPixbuf
from gvsbuild.projects.gettext import Gettext
from gvsbuild.projects.glib import GLib, GLibNetworking, GLibPyWrapper
from gvsbuild.projects.gobject_introspection import GObjectIntrospection
from gvsbuild.projects.graphene import Graphene
from gvsbuild.projects.gsettings_desktop_schemas import GSettingsDesktopSchemas
from gvsbuild.projects.gstreamer import (
    GstPluginsBad,
    GstPluginsBase,
    GstPluginsGood,
    GstPython,
    GStreamer,
    Orc,
)
from gvsbuild.projects.gtk import Gtk2, Gtk3, Gtk4
from gvsbuild.projects.gtksourceview import GtkSourceView4, GtkSourceView5
from gvsbuild.projects.harfbuzz import Harfbuzz
from gvsbuild.projects.hicolor_icon_theme import HicolorIconTheme
from gvsbuild.projects.icu import Icu
from gvsbuild.projects.json_glib import JsonGLib
from gvsbuild.projects.jsonc import Jsonc
from gvsbuild.projects.leveldb import LevelDB
from gvsbuild.projects.lgi import Lgi
from gvsbuild.projects.libadwaita import Libadwaita
from gvsbuild.projects.libarchive import Libarchive
from gvsbuild.projects.libcroco import Libcroco
from gvsbuild.projects.libcurl import Libcurl
from gvsbuild.projects.libepoxy import Libepoxy
from gvsbuild.projects.libffi import Libffi
from gvsbuild.projects.libgxps import Libgxps
from gvsbuild.projects.libjpeg_turbo import LibjpegTurbo
from gvsbuild.projects.libmicrohttpd import Libmicrohttpd
from gvsbuild.projects.libpng import Libpng
from gvsbuild.projects.libpsl import Libpsl
from gvsbuild.projects.librsvg import Librsvg
from gvsbuild.projects.libsoup import Libsoup2, Libsoup3
from gvsbuild.projects.libssh import Libssh, Libssh2
from gvsbuild.projects.libtiff import Libtiff4
from gvsbuild.projects.libuv import Libuv
from gvsbuild.projects.libvpx import Libvpx
from gvsbuild.projects.libxml2 import Libxml2
from gvsbuild.projects.libyuv import Libyuv
from gvsbuild.projects.libzip import Libzip
from gvsbuild.projects.lmdb import Lmdb
from gvsbuild.projects.luajit import Luajit
from gvsbuild.projects.lz4 import Lz4
from gvsbuild.projects.mit_kerberos import Kerberos
from gvsbuild.projects.nghttp2 import Nghttp2
from gvsbuild.projects.openh264 import OpenH264
from gvsbuild.projects.openssl import OpenSSL
from gvsbuild.projects.opus import Opus
from gvsbuild.projects.pango import Pango
from gvsbuild.projects.pixman import Pixman
from gvsbuild.projects.pkgconf import PkgConf
from gvsbuild.projects.portaudio import Portaudio
from gvsbuild.projects.protobuf import Protobuf, ProtobufC
from gvsbuild.projects.pycairo import Pycairo
from gvsbuild.projects.pygobject import PyGObject
from gvsbuild.projects.sqlite import SQLite
from gvsbuild.projects.win_iconv import WinIconv
from gvsbuild.projects.wing import Wing
from gvsbuild.projects.x264 import X264
from gvsbuild.projects.zlib import Zlib



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libmicrohttpd(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "libmicrohttpd",
            archive_url="http://ftp.gnu.org/gnu/libmicrohttpd/libmicrohttpd-0.9.54.tar.gz",
            hash="bcc721895d4a114b0548a39d2241c35caacb9e2e072d40e11b55c60e3d5ddcbe",
            patches=["001-remove-postsample.patch"],
        )

    def build(self):
        if self.builder.opts.configuration == "debug":
            configuration = "debug-dll"
        else:
            configuration = "release-dll"
        td = self.exec_msbuild_gen(
            r"w32", "libmicrohttpd.sln", configuration=configuration
        )
        base_dir = os.path.join("w32", td)

        debug_option = r"_d" if self.builder.opts.configuration == "debug" else ""
        rel_dir = ".\\" + base_dir + r"\Output"
        if not self.builder.x86:
            rel_dir += r"\x64"

        self.push_location(rel_dir)
        self.install(r"microhttpd.h include")
        self.install(f"libmicrohttpd-dll{debug_option}.lib lib")
        self.install(f"libmicrohttpd-dll{debug_option}.dll bin")
        self.install(f"libmicrohttpd-dll{debug_option}.pdb bin")
        self.install(f"hellobrowser-dll{debug_option}.exe bin")
        self.pop_location()

        self.install(r".\COPYING share\doc\libmicrohttpd")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class AdwaitaIconTheme(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "adwaita-icon-theme",
            archive_url="https://download.gnome.org/sources/adwaita-icon-theme/43/adwaita-icon-theme-43.tar.xz",
            hash="2e3ac77d32a6aa5554155df37e8f0a0dd54fc5a65fd721e88d505f970da32ec6",
            dependencies=[
                "librsvg",
                "python",
            ],
        )

    def build(self):
        # Create the destination dir, before the build
        os.makedirs(
            os.path.join(self.builder.gtk_dir, "share", "icons", "Adwaita"),
            exist_ok=True,
        )

        self.push_location(r".\win32")
        self.exec_vs(
            r'nmake /nologo /f adwaita-msvc.mak CFG=%(configuration)s PYTHON="%(python_dir)s\python.exe" PREFIX="%(gtk_dir)s"',
            add_path=os.path.join(self.builder.opts.msys_dir, "usr", "bin"),
        )
        self.exec_vs(
            r'nmake /nologo /f adwaita-msvc.mak install CFG=%(configuration)s PYTHON="%(python_dir)s\python.exe" PREFIX="%(gtk_dir)s"',
            add_path=os.path.join(self.builder.opts.msys_dir, "usr", "bin"),
        )
        self.pop_location()

        self.install(r".\COPYING_CCBYSA3 share\doc\adwaita-icon-theme")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add
from gvsbuild.utils.simple_ui import log
from gvsbuild.utils.utils import python_find_libs_dir


@project_add
class GObjectIntrospection(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gobject-introspection",
            archive_url="https://download.gnome.org/sources/gobject-introspection/1.74/gobject-introspection-1.74.0.tar.xz",
            hash="347b3a719e68ba4c69ff2d57ee2689233ea8c07fc492205e573386779e42d653",
            dependencies=[
                "ninja",
                "meson",
                "msys2",
                "pkg-config",
                "glib",
            ],
            # https://gitlab.gnome.org/GNOME/gobject-introspection/-/issues/427
            patches=["incorrect-giscanner-path.patch"],
        )

    def build(self):
        # For finding gobject-introspection.pc
        self.builder.mod_env("PKG_CONFIG_PATH", ".")
        # For finding & using girepository.lib/.dll
        self.builder.mod_env("LIB", r".\girepository")
        self.builder.mod_env("PATH", r".\girepository")
        # For linking the _giscanner.pyd extension module when using a virtualenv
        py_dir = Project.get_tool_path("python")
        py_libs = python_find_libs_dir(py_dir)
        if py_libs:
            log.debug(f"Python library path is [{py_libs}]")
            self.builder.mod_env("LIB", py_libs, prepend=False)

        Meson.build(
            self,
            meson_params="-Dpython=%s\\python.exe -Dcairo_libname=cairo-gobject.dll"
            % (py_dir,),
        )



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Librsvg(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "librsvg",
            archive_url="https://download.gnome.org/sources/librsvg/2.54/librsvg-2.54.4.tar.xz",
            hash="ea152a243f6a43c0e036a28c70de3fcbcdea5664c6811c78592bc229ecc24833",
            dependencies=[
                "cargo",
                "cairo",
                "pango",
                "gdk-pixbuf",
            ],
            patches=[],
        )
        if Project.opts.enable_gi:
            self.add_dependency("gobject-introspection")

    def build(self):
        self.builder.mod_env("INCLUDE", "include\\cairo", add_gtk=True)

        b_dir = f"{self.builder.working_dir}\\{self.name}\\win32"

        cmd = f'nmake -f makefile.vc CFG={self.builder.opts.configuration} PREFIX={self.builder.gtk_dir} PYTHON={Project.get_tool_executable("python")} install'

        if Project.opts.enable_gi:
            cmd += " INTROSPECTION=1"

        self.push_location(b_dir)
        self.exec_vs(cmd)
        self.pop_location()

        self.install(r".\COPYING.LIB share\doc\librsvg")

    def post_install(self):
        self.exec_cmd(r"%(gtk_dir)s\bin\gdk-pixbuf-query-loaders.exe --update-cache")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#  Copyright (C) 2017 - Daniele Forghieri
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Base group class, from the project one, as a placeholder to build more than
one project from a single one."""

from .base_project import Project, ProjectType


class Group(Project):
    def __init__(self, name, **kwargs):
        Project.__init__(self, name, **kwargs)

    def unpack(self):
        # We don't have to do anything
        pass

    def build(self):
        pass

    def export(self):
        pass


def group_add(cls):
    """Class decorator to add the newly created Group class to the global
    projects/tools/groups list."""
    Project.register(cls, ProjectType.GROUP)
    return cls



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class PkgConf(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "pkg-config",
            prj_dir="pkgconf",
            archive_url="https://distfiles.dereferenced.org/pkgconf/pkgconf-1.8.0.tar.gz",
            hash="d7b6fdb522d81c11f5a0e0a0629a9f5480809ec90e595058674c1517822dfb8c",
            dependencies=["ninja", "meson"],
            patches=["0001-vs2013.patch"],
        )
        self.add_param("-Dtests=false")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\pkgconf")

    def post_install(self):
        self.exec_cmd(
            r"copy %(gtk_dir)s\bin\pkgconf.exe %(gtk_dir)s\bin\pkg-config.exe"
        )



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Libssh(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libssh",
            archive_url="https://www.libssh.org/files/0.9/libssh-0.9.3.tar.xz",
            hash="2c8b5f894dced58b3d629f16f3afa6562c20b4bdc894639163cf657833688f0c",
            dependencies=["zlib", "openssl", "cmake", "ninja"],
        )

    def build(self):
        CmakeProject.build(self, cmake_params="-DWITH_ZLIB=ON", use_ninja=True)
        self.install(r".\COPYING share\doc\libssh")


@project_add
class Libssh2(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libssh2",
            archive_url="https://www.libssh2.org/download/libssh2-1.8.0.tar.gz",
            hash="39f34e2f6835f4b992cafe8625073a88e5a28ba78f83e8099610a7b3af4676d4",
            dependencies=[
                "cmake",
                "ninja",
            ],
        )

    def build(self):
        CmakeProject.build(self, cmake_params="-DWITH_ZLIB=ON", use_ninja=True)
        self.install(r".\COPYING share\doc\libssh2")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Pixman(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "pixman",
            archive_url="http://cairographics.org/releases/pixman-0.40.0.tar.gz",
            hash="6d200dec3740d9ec4ec8d1180e25779c00bc749f94278c8b9021f5534db223fc",
        )

    def build(self):
        optimizations = "SSE2=on SSSE3=on" + (
            " MMX=off" if self.builder.x64 else " MMX=on"
        )

        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        self.exec_vs(
            r"make -f Makefile.win32 pixman CFG=%(configuration)s " + optimizations,
            add_path=add_path,
        )

        self.install(r".\pixman\%(configuration)s\pixman-1.lib lib")

        self.install(r".\pixman\pixman.h include\pixman-1")
        self.install(r".\pixman\pixman-version.h include\pixman-1")

        self.install(r".\COPYING share\doc\pixman")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Default tools used to build the various projects."""

import os
import shutil
import subprocess
import sys

from .utils.base_expanders import extract_exec
from .utils.base_project import Project
from .utils.base_tool import Tool, tool_add
from .utils.simple_ui import log


@tool_add
class ToolCargo(Tool):
    def __init__(self):
        Tool.__init__(
            self,
            "cargo",
            archive_url="https://win.rustup.rs/x86_64",
            archive_file_name="rustup-init.exe",
            exe_name="cargo.exe",
        )

    def load_defaults(self):
        Tool.load_defaults(self)
        self.tool_path = os.path.join(self.build_dir, "bin")
        self.full_exe = os.path.join(self.tool_path, "cargo.exe")

        self.add_extra_env("RUSTUP_HOME", self.build_dir)
        self.add_extra_env("CARGO_HOME", self.build_dir)

    def unpack(self):
        env = os.environ.copy()
        env["RUSTUP_HOME"] = self.build_dir
        env["CARGO_HOME"] = self.build_dir

        rustup = os.path.join(self.build_dir, "bin", "rustup.exe")

        subprocess.check_call(
            f"{self.archive_file} --no-modify-path -y", shell=True, env=env
        )

        # add supported targets
        subprocess.check_call(
            f"{rustup} target add x86_64-pc-windows-msvc", shell=True, env=env
        )

        subprocess.check_call(
            f"{rustup} target add i686-pc-windows-msvc", shell=True, env=env
        )

        # switch to the right target
        subprocess.check_call(
            f'{rustup} default stable-{"i686" if self.opts.x86 else "x86_64"}-pc-windows-msvc',
            env=env,
        )

        self.mark_deps = True


@tool_add
class ToolCmake(Tool):
    def __init__(self):
        self.version = "3.24.2"
        Tool.__init__(
            self,
            "cmake",
            archive_url=f"https://github.com/Kitware/CMake/releases/download/v{self.version}/cmake-{self.version}-windows-x86_64.zip",
            hash="6af30354eecbb7113b0f0142d13c03d21abbc9f4dbdcddaf88df1f9ca1bc4d6f",
            dir_part=f"cmake-{self.version}-windows-x86_64",
        )

    def load_defaults(self):
        Tool.load_defaults(self)
        self.tool_path = os.path.join(self.build_dir, "bin")
        self.full_exe = os.path.join(self.tool_path, "cmake.exe")

    def unpack(self):
        self.mark_deps = extract_exec(
            self.archive_file,
            self.opts.tools_root_dir,
            dir_part=self.dir_part,
            check_file=self.full_exe,
            check_mark=True,
        )


@tool_add
class ToolMeson(Tool):
    def __init__(self):
        self.version = "0.63.2"
        Tool.__init__(
            self,
            "meson",
            archive_url=f"https://github.com/mesonbuild/meson/archive/refs/tags/{self.version}.tar.gz",
            archive_file_name=f"meson-{self.version}.tar.gz",
            hash="023a3f7c74e68991154c3205a6975705861eedbf8130e013d15faa1df1af216e",
            dependencies=[
                "python",
            ],
            dir_part=f"meson-{self.version}",
            exe_name="meson.py",
        )

    def unpack(self):
        self.mark_deps = extract_exec(
            self.archive_file,
            self.builder.opts.tools_root_dir,
            dir_part=self.dir_part,
            check_file=self.full_exe,
            check_mark=True,
        )


@tool_add
class ToolMsys2(Tool):
    def __init__(self):
        Tool.__init__(self, "msys2")

    def load_defaults(self):
        Tool.load_defaults(self)
        self.tool_path = os.path.join(self.opts.msys_dir, "usr", "bin")

    def unpack(self):
        self.tool_mark()

    def get_path(self):
        # We always put msys at the end of path
        return None, self.tool_path


@tool_add
class ToolNasm(Tool):
    def __init__(self):
        Tool.__init__(
            self,
            "nasm",
            archive_url="https://github.com/wingtk/gvsbuild/releases/download/nasm-2.15.05/nasm-2.15.05-win64.zip",
            hash="f5c93c146f52b4f1664fa3ce6579f961a910e869ab0dae431bd871bdd2584ef2",
            dir_part="nasm-2.15.05",
            exe_name="nasm.exe",
        )

    def unpack(self):
        # We download directly the exe file so we copy it on the tool directory ...
        self.mark_deps = extract_exec(
            self.archive_file,
            self.builder.opts.tools_root_dir,
            dir_part=self.dir_part,
            check_file=self.full_exe,
            force_dest=self.full_exe,
            check_mark=True,
        )


@tool_add
class ToolNinja(Tool):
    def __init__(self):
        self.version = "1.11.0"
        Tool.__init__(
            self,
            "ninja",
            archive_url=f"https://github.com/ninja-build/ninja/releases/download/v{self.version}/ninja-win.zip",
            archive_file_name=f"ninja-win-{self.version}.zip",
            hash="d0ee3da143211aa447e750085876c9b9d7bcdd637ab5b2c5b41349c617f22f3b",
            dir_part=f"ninja-{self.version}",
            exe_name="ninja.exe",
        )

    def unpack(self):
        self.mark_deps = extract_exec(
            self.archive_file, self.build_dir, check_file=self.full_exe, check_mark=True
        )


@tool_add
class ToolNuget(Tool):
    def __init__(self):
        self.version = "6.3.0"
        Tool.__init__(
            self,
            "nuget",
            archive_url=f"https://dist.nuget.org/win-x86-commandline/v{self.version}/nuget.exe",
            archive_file_name=f"nuget-{self.version}.exe",
            hash="38257f945b3662f5c81d646f2cd940365642e2c4941b34c0e5ae3f4281f9bd2d",
            dir_part=f"nuget-{self.version}",
            exe_name="nuget.exe",
        )

    def unpack(self):
        # Download the exe file and copy it on the tool directory
        self.mark_deps = extract_exec(
            self.archive_file,
            self.build_dir,
            check_file=self.full_exe,
            force_dest=self.full_exe,
            check_mark=True,
        )


@tool_add
class ToolPerl(Tool):
    def __init__(self):
        Tool.__init__(
            self,
            "perl",
            archive_url="https://github.com/wingtk/gtk-win32/releases/download/Perl-5.20/perl-5.20.0-x64.tar.xz",
            hash="05e01cf30bb47d3938db6169299ed49271f91c1615aeee5649174f48ff418c55",
            dir_part="perl-5.20.0",
        )

    def load_defaults(self):
        Tool.load_defaults(self)
        # Set the builder object to point to the path to use, when we need to pass directly the executable to *make
        self.base_dir = os.path.join(self.build_dir, "x64")
        # full path, added to the environment when needed
        self.tool_path = os.path.join(self.base_dir, "bin")
        self.full_exe = os.path.join(self.tool_path, "perl.exe")

    def unpack(self):
        self.mark_deps = extract_exec(
            self.archive_file, self.build_dir, check_file=self.full_exe, check_mark=True
        )

    def get_base_dir(self):
        return self.base_dir


@tool_add
class ToolPython(Tool):
    def __init__(self):
        Tool.__init__(
            self,
            "python",
            dependencies=[
                "nuget",
            ],
        )

    def setup(self, install):
        """Using nuget install, locally, the specified version of python."""
        version = self.opts.python_ver
        # Get the last version we ask
        if version == "3.7":
            version = "3.7.14"
        elif version == "3.8":
            version = "3.8.14"
        elif version == "3.9":
            version = "3.9.14"
        elif version == "3.10":
            version = "3.10.7"

        name = "pythonx86" if self.opts.x86 else "python"
        t_id = f"{name}.{version}"
        dest_dir = os.path.join(self.opts.tools_root_dir, t_id)
        # directory to use for the .exe
        self.tool_path = os.path.join(dest_dir, "tools")
        self.full_exe = os.path.join(self.tool_path, "python.exe")

        if install:
            # see if it's already ok
            rd_file = ""
            try:
                with open(os.path.join(dest_dir, ".wingtk-extracted-file")) as fi:
                    rd_file = fi.readline().strip()
            except OSError:
                pass

            if rd_file == t_id:
                # Ok, exit
                log.log(f"Skipping python setup on '{dest_dir}'")
                # We don't rebuild the projects that depend on this
                return False

            # nuget
            nuget = Project.get_tool_executable("nuget")
            # Install python
            cmd = f"{nuget} install {name} -Version {version} -OutputDirectory {self.opts.tools_root_dir}"

            subprocess.check_call(cmd, shell=True)
            py = os.path.join(self.tool_path, "python.exe")

            # Update pip
            cmd = f"{py} -m pip install --upgrade pip setuptools wheel build --no-warn-script-location"
            subprocess.check_call(cmd, shell=True)

            python3 = os.path.join(self.tool_path, "python3.exe")
            if not os.path.exists(python3):
                # We create a python3.exe file so meson find our python and not some other
                # lying around (e.g. one from the Visual Studio installation ...)
                log.log(f"Create python3 copy on '{dest_dir}'")
                shutil.copy(self.full_exe, python3)

            # Mark that we have done all
            with open(os.path.join(dest_dir, ".wingtk-extracted-file"), "wt") as fo:
                fo.write(f"{t_id}\n")

        return True

    def load_defaults(self):
        Tool.load_defaults(self)
        self.setup(False)

    def unpack(self):
        if self.opts._load_python:
            # Get python version
            self.mark_deps = self.setup(True)
        else:
            self.tool_path = self.opts.python_dir or os.path.dirname(sys.executable)
            self.full_exe = os.path.join(self.tool_path, "python.exe")
            self.mark_deps = False


@tool_add
class ToolYasm(Tool):
    def __init__(self):
        Tool.__init__(
            self,
            "yasm",
            archive_url="http://www.tortall.net/projects/yasm/releases/yasm-1.3.0-win64.exe",
            hash="d160b1d97266f3f28a71b4420a0ad2cd088a7977c2dd3b25af155652d8d8d91f",
            dir_part="yasm-1.3.0",
            exe_name="yasm.exe",
        )

    def unpack(self):
        # We download directly the exe file so we copy it on the tool directory ...
        self.mark_deps = extract_exec(
            self.archive_file,
            self.build_dir,
            check_file=self.full_exe,
            force_dest=self.full_exe,
            check_mark=True,
        )


@tool_add
class ToolGo(Tool):
    def __init__(self):
        Tool.__init__(
            self,
            "go",
            archive_url="https://go.dev/dl/go1.18.3.windows-amd64.zip",
            hash="9c46023f3ad0300fcfd1e62f2b6c2dfd9667b1f2f5c7a720b14b792af831f071",
            dir_part="go-1.18",
        )

    def load_defaults(self):
        Tool.load_defaults(self)
        self.tool_path = os.path.join(self.build_dir, "bin")
        self.full_exe = os.path.join(self.tool_path, "go.exe")

    def unpack(self):
        # We download directly the exe file so we copy it on the tool directory ...
        self.mark_deps = extract_exec(
            self.archive_file,
            self.build_dir,
            check_file=self.full_exe,
            check_mark=True,
            strip_one=True,
        )



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Nghttp2(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "nghttp2",
            archive_url="https://github.com/nghttp2/nghttp2/releases/download/v1.45.1/nghttp2-1.45.1.tar.xz",
            hash="abdc4addccadbc7d89abe27c4d6427d78e57d139f69c1f45749227393c68bf79",
            dependencies=[
                "cmake",
                "zlib",
                "ninja",
            ],
            patches=["0001-Define-ssize_t-if-not-defined.patch"],
        )

    def build(self):
        CmakeProject.build(self, use_ninja=True)

        self.install(r".\COPYING share\doc\nghttp2")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Pycairo(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "pycairo",
            archive_url="https://github.com/pygobject/pycairo/releases/download/v1.21.0/pycairo-1.21.0.tar.gz",
            hash="251907f18a552df938aa3386657ff4b5a4937dde70e11aa042bc297957f4b74b",
            dependencies=["cairo", "python"],
        )

    def build(self):
        cairo_inc = Path(self.builder.gtk_dir) / "include" / "cairo"
        self.builder.mod_env("INCLUDE", str(cairo_inc))
        self.exec_vs(r"%(python_dir)s\python.exe -m build")
        dist_dir = Path(self.build_dir) / "dist"
        for path in dist_dir.rglob("*.whl"):
            self.exec_vs(r"%(python_dir)s\python.exe -m pip install " + str(path))
        if self.builder.opts.py_wheel:
            self.install_dir("dist", "python")
        self.install(r".\COPYING share\doc\pycairo")
        self.install(r".\COPYING-LGPL-2.1 share\doc\pycairo")
        self.install(r".\COPYING-MPL-1.1 share\doc\pycairo")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Command line parser."""

import argparse
import os
import re
import sys
from pathlib import Path
from typing import Any, Tuple, Union

from .base_project import Options, Project, ProjectType
from .builder import Builder
from .simple_ui import log
from .utils import ordered_set


def get_options(args):
    opts = Options()

    opts.verbose = args.verbose
    opts.debug = args.debug
    opts.platform = args.platform
    opts.configuration = getattr(args, "configuration", "release")
    opts.build_dir = args.build_dir
    opts.archives_download_dir = args.archives_download_dir
    opts.export_dir = args.export_dir
    opts.patches_root_dir = args.patches_root_dir
    opts.tools_root_dir = args.tools_root_dir
    opts.vs_ver = args.vs_ver
    opts.vs_install_path = args.vs_install_path
    opts.win_sdk_ver = args.win_sdk_ver
    opts.net_target_framework = args.net_target_framework
    opts.net_target_framework_version = args.net_target_framework_version
    opts.python_dir = args.python_dir
    opts.msys_dir = args.msys_dir
    opts.clean = args.clean
    opts.msbuild_opts = args.msbuild_opts
    opts.use_env = args.use_env
    opts.no_deps = args.no_deps
    opts.check_hash = args.check_hash
    opts.skip = args.skip
    opts.make_zip = args.make_zip
    opts.zip_continue = args.zip_continue
    opts.from_scratch = args.from_scratch
    opts.keep_tools = args.keep_tools
    opts.fast_build = args.fast_build
    opts.keep = args.keep
    opts.clean_built = args.clean_built
    opts.py_wheel = args.py_wheel
    opts.enable_gi = args.enable_gi
    opts.ffmpeg_enable_gpl = args.ffmpeg_enable_gpl
    opts.log_size = args.log_size
    opts.log_single = args.log_single
    opts.cargo_opts = args.cargo_opts
    opts.ninja_opts = args.ninja_opts
    opts.python_ver = args.python_ver
    opts.same_python = args.same_python
    opts.capture_out = args.capture_out
    opts.print_out = args.print_out
    opts.git_expand_dir = args.git_expand_dir

    # active the log
    log.configure(os.path.join(opts.build_dir, "logs"), opts)

    if opts.make_zip and opts.no_deps:
        log.error_exit("Options --make-zip and --no-deps are not compatible")

    if not opts.archives_download_dir:
        opts.archives_download_dir = os.path.join(args.build_dir, "src")
    if not opts.git_expand_dir:
        opts.git_expand_dir = os.path.join(opts.archives_download_dir, "git-exp")
    if not opts.export_dir:
        opts.export_dir = os.path.join(args.build_dir, "export")
    if not opts.patches_root_dir:
        opts.patches_root_dir = str(Path(__file__).parent.parent / "patches")
    prop_file = os.path.join(opts.patches_root_dir, "stack.props")
    if not os.path.isfile(prop_file):
        log.error_exit(
            "Missing 'stack.props' file on directory '%s'.\nWrong or missing --patches-root-dir option?"
            % (opts.patches_root_dir,)
        )

    if not opts.tools_root_dir:
        opts.tools_root_dir = os.path.join(args.build_dir, "tools")

    if opts.python_dir is None and not opts.same_python:
        opts._load_python = True

    opts.projects = args.project
    Project.opts = opts
    # now add the tools/projects/groups
    Project.add_all()

    for p in opts.projects:
        if p not in Project.get_names():
            log.error_exit(
                p
                + " is not a valid project name, available projects are:\n\t"
                + "\n\t".join(Project.get_names())
            )

    return opts


def __get_projects_to_build(opts):
    to_build = ordered_set()
    if opts._load_python:
        # We use nuget to download & install the python needed for the build so we put it at the beginning
        opts.projects.insert(0, "python")

    for name in opts.projects:
        p = Project.get_project(name)
        if not opts.no_deps:
            for dep in p.all_dependencies:
                to_build.add(dep)
        to_build.add(p)
        if opts.clean_built:
            p.clean = True

    # See if we need to drop some project
    if opts.skip:
        to_skip = opts.skip.split(",")
        for s in to_skip:
            if s not in Project.get_names():
                log.error_exit(
                    s
                    + " is not a valid project name, available projects are:\n\t"
                    + "\n\t".join(Project.get_names())
                )

            p = Project.get_project(s)
            if p in to_build:
                log.debug(f"Dropped project {s}")
                to_build.remove(p)
    return to_build


def do_build(args):
    opts = get_options(args)
    if log.debug_on():
        log.debug("Options are:")
        for co in sorted(opts.__dict__.keys()):
            v = opts.__dict__[co]
            pv = f"'{v}'" if type(v) is str else repr(v)
            log.message_indent(f"'{co}': {pv}, ")
    builder = Builder(opts)
    builder.preprocess()

    to_build = __get_projects_to_build(opts)
    if not to_build:
        log.error_exit("nothing to do")
    log.debug(f"building {[p.name for p in to_build]}")

    builder.build(to_build)


def get_project_by_type(prj_type):
    return [
        (project.name, project.version)
        for project in Project._projects
        if project.type == prj_type
    ]


def do_list(args):
    def do_list_type(prj_type, desc):
        projects = get_project_by_type(prj_type)
        if projects:
            projects.sort()

            print(f"{desc}:")
            for project in projects:
                print(f"\t{project[0]:<{Project.name_len}} {project[1]}")

    # now add the tools/projects/groups
    Project.add_all()
    do_list_type(ProjectType.TOOL, "Available tools")
    do_list_type(ProjectType.PROJECT, "Available projects")
    do_list_type(ProjectType.GROUP, "Available groups")
    do_list_type(ProjectType.IGNORE, "Developer project(s)")
    sys.exit(0)


def separate_name_and_major_version(name: str) -> Tuple[Union[str, Any], ...]:
    # Exceptions where ending with a simple digit is part of the library name
    if name in {"nghttp2", "ssh2", "libxml2", "libtiff-4"}:
        return name, None
    # https://regex101.com/r/1c4iLx/2
    match = re.search(r"([a-z-]*\d{3}|[a-z-]*\d{0})(\d$)?", name)
    return match.group(1, 2) if match else (None, None)


def do_outdated(args):
    try:
        import lastversion
        from packaging import version
    except ImportError:
        print("Please pip install .[outdated] in your Python environment")
        sys.exit(0)

    Project.add_all()
    projects = get_project_by_type(ProjectType.PROJECT)
    projects.extend(get_project_by_type(ProjectType.TOOL))
    print("Looking for projects and tools that are out-of-date, please submit a PR!")
    print(f"\t{'Project Name':<{Project.name_len}} {'Current':<45} {'Latest':<45}")
    try:
        for project in projects:
            # glib-py-wrapper and check-libs are vendored in gvsbuild
            if project[0] in ("glib-py-wrapper", "check-libs") or not project[1]:
                continue
            name_and_major = separate_name_and_major_version(project[0])
            repos = {
                "adwaita-icon-theme": "https://gitlab.gnome.org/GNOME/adwaita-icon-theme",
                "atk": "https://gitlab.gnome.org/GNOME/atk",
                "boringssl": "https://github.com/google/boringssl",
                "clutter": "https://gitlab.gnome.org/GNOME/clutter",
                "cogl": "https://gitlab.gnome.org/Archive/cogl",
                "emeus": "https://github.com/ebassi/emeus",
                "fontconfig": "https://gitlab.freedesktop.org/fontconfig/fontconfig",
                "freetype": "https://gitlab.freedesktop.org/freetype/freetype",
                "gdk-pixbuf": "https://gitlab.gnome.org/GNOME/gdk-pixbuf",
                "gettext": "autotools-mirror/gettext",
                "glib": "https://gitlab.gnome.org/GNOME/glib",
                "glib-networking": "https://gitlab.gnome.org/GNOME/glib-networking",
                "glib-py-wrapper": "https://gitlab.gnome.org/GNOME/glib-py-wrapper",
                "gobject-introspection": "https://gitlab.gnome.org/GNOME/gobject-introspection",
                "graphene": "ebassi/graphene",
                "gsettings-desktop-schemas": "https://gitlab.gnome.org/GNOME/gsettings-desktop-schemas",
                "gtk3": "https://gitlab.gnome.org/GNOME/gtk",
                "gtk4": "https://gitlab.gnome.org/GNOME/gtk",
                "gtksourceview4": "https://gitlab.gnome.org/GNOME/gtksourceview",
                "gtksourceview5": "https://gitlab.gnome.org/GNOME/gtksourceview",
                "hicolor-icon-theme": "https://gitlab.freedesktop.org/xdg/default-icon-theme",
                "json-glib": "https://gitlab.gnome.org/GNOME/json-glib",
                "libcroco": "https://gitlab.gnome.org/Archive/libcroco",
                "libcurl": "https://github.com/curl/curl",
                "libmicrohttpd": "https://github.com/Karlson2k/libmicrohttpd",
                "libsoup2": "https://gitlab.gnome.org/GNOME/libsoup",
                "libsoup3": "https://gitlab.gnome.org/GNOME/libsoup",
                "libssh": "libssh/libssh-mirror",
                "libssh2": "libssh2/libssh2",
                "libtiff-4": "https://gitlab.com/libtiff/libtiff",
                "libxml2": "https://gitlab.gnome.org/GNOME/libxml2",
                "orc": "https://gitlab.freedesktop.org/gstreamer/orc",
                "pango": "https://gitlab.gnome.org/GNOME/pango",
                "pixman": "https://gitlab.freedesktop.org/pixman/pixman",
                "pkg-config": "pkgconf",
                "pygobject": "https://gitlab.gnome.org/GNOME/pygobject",
                "wing": "https://gitlab.gnome.org/GNOME/wing",
            }
            try:
                repo = repos.get(project[0], name_and_major[0])
                if name_and_major[1]:
                    latest_version = lastversion.latest(
                        repo=repo, major=name_and_major[1]
                    )
                else:
                    latest_version = lastversion.latest(
                        repo=repo,
                    )
                if not latest_version:
                    print(
                        f"\t{project[0]:<{Project.name_len}} {project[1]:<45} {'No release found':<45}"
                    )
                elif version.parse(str(latest_version)) > version.parse(project[1]):
                    print(
                        f"\t{project[0]:<{Project.name_len}} {project[1]:<45} {str(latest_version):<45}"
                    )
            except version.InvalidVersion:
                print(f"Project {project[0]} does not have a valid version")
    except lastversion.utils.ApiCredentialsError:
        print("Set GITHUB_API_TOKEN=xxxxxxxxxxxxxxx environmental variable")


def create_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description="Jhbuild-like system for Windows to build Gtk and friends",
        fromfile_prefix_chars="@",
        epilog="""
Examples:
    gvsbuild build libpng libffi
        Build libpng, libffi, and their dependencies (zlib).

    gvsbuild build --no-deps glib
        Build glib only.

    gvsbuild build --skip gtk3,pycairo,pygobject all
        Build everything except gtk3, pycairo, and pygobject
    """,
    )

    # ==============================================================================
    # Global options
    # ==============================================================================

    parser.add_argument(
        "-v",
        "--verbose",
        default=False,
        action="store_true",
        help="Print lots of stuff.",
    )
    parser.add_argument(
        "-d",
        "--debug",
        default=False,
        action="store_true",
        help="Print even more stuff.",
    )

    subparsers = parser.add_subparsers()

    # ==============================================================================
    # build
    # ==============================================================================

    p_build = subparsers.add_parser("build", help="build project(s)")
    p_build.set_defaults(func=do_build)

    p_build.add_argument(
        "-p",
        "--platform",
        default="x64",
        choices=["x86", "x64"],
        help="Platform to build for, x86 or x64. Default is x64.",
    )
    p_build.add_argument(
        "-c",
        "--configuration",
        default="release",
        choices=["release", "debug"],
        help="Configuration to build, release or debug. Default is release.",
    )
    p_build.add_argument(
        "--build-dir",
        default=r"C:\gtk-build",
        help="The directory where the sources will be downloaded and built.",
    )
    p_build.add_argument(
        "--msys-dir",
        default=r"C:\Msys64",
        help="The directory where you installed msys2.",
    )
    p_build.add_argument(
        "--archives-download-dir",
        help="The directory to download the source archives to. It will be created. "
        + "If a source archive already exists here, it won't be downloaded again. "
        + "Default is $(build-dir)\\src.",
    )
    p_build.add_argument(
        "--export-dir",
        help="The directory to export the source archives to. It will be created. "
        + "It creates an archive with the source code and any possible patch. "
        + "Default is $(build-dir)\\export.",
    )
    p_build.add_argument(
        "--patches-root-dir",
        help="The directory where you checked out https://github.com/wingtk/gvsbuild.git. Default is $(build-dir)\\github\\gvsbuild.",
    )
    p_build.add_argument(
        "--tools-root-dir",
        help="The directory where to install the downloaded tools. Default is $(build-dir)\\tools.",
    )
    p_build.add_argument(
        "--vs-ver",
        default="17",
        help="Visual Studio version 12 (vs2013), 14 (vs2015), 15 (vs2017), 16 (vs2019), 17 (vs2022). Default is 17.",
    )
    p_build.add_argument(
        "--vs-install-path",
        help=r"The directory where you installed Visual Studio. Default is 'C:\Program Files (x86)\Microsoft Visual Studio $(vs-ver).0' (for vs-ver <= 14) "
        + "or 'C:\\Program Files (x86)\\Microsoft Visual Studio\\20xx' (2017 for vs-ver 15, 2019 for vs-ver 16, ...). "
        "If not set for the vs2017 version the script look automatically under Professional, BuildTools, Enterprise, Community and Preview sub directory until it finds the startup batch.",
    )
    p_build.add_argument(
        "--win-sdk-ver",
        default=None,
        help=r"The windows sdk version to use for building, used to initialize the Visual Studio build environment. "
        + "It can be 8.1 (for windows 8 compatibility) or 10.0.xxxxx.0, where xxxxx, at the moment, can be 10150, 10240, 10586, 14393, 15063 "
        + "16299, 17134 or 17763 "
        + "depending on the VS version / installation's options. "
        + "If you don't specify one the scripts tries to locate the used one to pass the value to the msbuild command.",
    )
    p_build.add_argument(
        "--net-target-framework",
        default=None,
        help=".net target framework. If set then TargetFrameworks parameter is passed down to msbuild with the specific target. i.e net45",
    )
    p_build.add_argument(
        "--net-target-framework-version",
        default=None,
        help=".net target framework version. If set then TargetFrameworkVersion parameter is passed down to msbuild with the specific version. i.e v4.6.2",
    )
    p_build.add_argument(
        "--python-ver",
        default="3.10",
        help="Python version to download and use for the build (3.10, 3.9, 3.8, 3.7, or the exact one, 3.10.2 or 3.8.0-a3.",
    )
    p_build.add_argument(
        "--python-dir",
        default=None,
        help="The directory containing the python you want to use for the build of the projects (not the one used to run the script).",
    )
    p_build.add_argument(
        "--same-python",
        default=False,
        action="store_true",
        help="Use for the build the same python used to run this script",
    )

    p_build.add_argument(
        "--check-hash",
        default=False,
        action="store_true",
        help="Only check hashes of downloaded archive(s), no build",
    )
    p_build.add_argument(
        "--clean",
        default=False,
        action="store_true",
        help="Build the project(s) from scratch",
    )
    p_build.add_argument(
        "--clean-built",
        default=False,
        action="store_true",
        help="Clean before the build only the project(s) asked on the command line, not all the ones to build (via a dependency)",
    )
    p_build.add_argument(
        "--no-deps",
        default=False,
        action="store_true",
        help="Do not build dependencies of the selected project(s)",
    )

    p_build.add_argument(
        "--msbuild-opts", default="", help="Command line options to pass to msbuild."
    )
    p_build.add_argument(
        "--use-env",
        default=False,
        action="store_true",
        help="Use and keep the calling environment for LIB, LIBPATH, INCLUDE and PATH",
    )

    p_build.add_argument(
        "--skip",
        default="",
        help="A comma separated list of project(s) not to build. For dev-shell is a list of tool not to activate.",
    )

    p_build.add_argument(
        "--make-zip",
        default=False,
        action="store_true",
        help="Create singles zips of the projects built under $(build-dir)\\dist\\vsXXXX[-sdkVer]\\[platform]\\[configuration], "
        + "for example 'c:\\gtk-build\\dist\\vs2015-8.1\\win32\\release'. "
        + "NOTE: the destination dir (e.g. 'c:\\gtk-build\\gtk\\win32\\release') "
        + "will be cleared completely before the build!",
    )
    p_build.add_argument(
        "--zip-continue",
        default=False,
        action="store_true",
        help="Don't initialize the zip creation phase and keep the destination dir.",
    )
    p_build.add_argument(
        "--from-scratch",
        default=False,
        action="store_true",
        help="Start from scratch, deleting, before starting the build, the build and the "
        + "destination directories of the project for the current platform/configuration "
        + "setup (e.g. 'c:\\gtk-build\\build\\win32\\release' and 'c:\\gtk-build\\gtk\\win32\\release'  "
        + "and the common tools ('c:\\gtk-build\\tools')",
    )
    p_build.add_argument(
        "--keep-tools",
        default=False,
        action="store_true",
        help="Active only when used with --from-scratch, keep and don't delete the (common) tool directory.",
    )
    p_build.add_argument(
        "--fast-build",
        default=False,
        action="store_true",
        help="Don't build a project if it's already built and not updated."
        + "Note: you can have wrong results if you change only the patches or the script (updating the tarball or the git source is handled correctly)",
    )
    p_build.add_argument(
        "-k",
        "--keep",
        default=False,
        action="store_true",
        help="Continue the build even on errors, dropping the projects that depends on the failed ones",
    )
    p_build.add_argument(
        "--py-wheel",
        default=False,
        action="store_true",
        help="pycairo/pygobject: build also the wheel distribution format",
    )
    p_build.add_argument(
        "--enable-gi",
        default=False,
        action="store_true",
        help="Create, for the gtk stack, the .gir/.typelib files for gobject introspection",
    )
    p_build.add_argument(
        "--ffmpeg-enable-gpl",
        default=False,
        action="store_true",
        help="ffmpeg: build with the gpl libraries/modules",
    )
    p_build.add_argument(
        "--log-size",
        default=0,
        type=int,
        help="Maximum log size (in kilobytes) before restarting with a new file",
    )
    p_build.add_argument(
        "--log-single",
        default=False,
        action="store_true",
        help="Always start a new log file, with date & time",
    )
    p_build.add_argument(
        "--capture-out",
        default=False,
        action="store_true",
        help="Capture the output of the build process and put it in the log file.",
    )
    p_build.add_argument(
        "--print-out",
        default=False,
        action="store_true",
        help="With --capture-out acrive print the result of the commands also on stdout.",
    )
    p_build.add_argument(
        "--ninja-opts",
        default="",
        help="Command line options to pass to ninja, e.g. to limit the use (-j 2) or for debug purpouse.",
    )
    p_build.add_argument(
        "--cargo-opts", default="", help="Command line options to pass to cargo."
    )
    p_build.add_argument(
        "--git-expand-dir",
        default=None,
        help="The directory where the projects from git are expanded and updated.",
    )
    p_build.add_argument("project", nargs="+", help="Project(s) to build.")

    # ==============================================================================
    # list
    # ==============================================================================

    p_list = subparsers.add_parser("list", help="list available projects")
    p_list.set_defaults(func=do_list)

    # ==============================================================================
    # outdated
    # ==============================================================================

    p_outdated = subparsers.add_parser(
        "outdated", help="list out of date projects and tools"
    )
    p_outdated.set_defaults(func=do_outdated)

    return parser



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import NullExpander, Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GLib(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "glib",
            archive_url="https://download.gnome.org/sources/glib/2.72/glib-2.72.4.tar.xz",
            hash="8848aba518ba2f4217d144307a1d6cb9afcc92b54e5c13ac1f8c4d4608e96f0e",
            dependencies=["ninja", "meson", "pkg-config", "gettext", "libffi", "zlib"],
            patches=[
                "glib-package-installation-directory.patch",
                "0002-pcre-add-fallback-url.patch",
                "0001-Fix-global-and-local-variables-hidden-by-local-varia.patch",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\glib")


@project_add
class GLibNetworking(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "glib-networking",
            archive_url="https://download.gnome.org/sources/glib-networking/2.74/glib-networking-2.74.0.tar.xz",
            hash="1f185aaef094123f8e25d8fa55661b3fd71020163a0174adb35a37685cda613b",
            dependencies=["pkg-config", "ninja", "meson", "glib", "openssl"],
            patches=[
                "0001-Mark-strings-for-translation-and-translate-just-in-e.patch",
                "0002-tlslog-add-meson-config-setting-to-log-at-debug-leve.patch",
            ],
        )

    def build(self):
        Meson.build(self, meson_params="-Dgnutls=disabled -Dopenssl=enabled")
        self.install(r".\COPYING share\doc\glib-networking")
        self.install(r".\LICENSE_EXCEPTION share\doc\glib-networking")


@project_add
class GLibPyWrapper(NullExpander, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "glib-py-wrapper",
            dependencies=["glib"],
            version="0.1.0",
        )

    def build(self):
        Meson.build(self)



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add
from gvsbuild.utils.utils import file_replace


@project_add
class Libcurl(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libcurl",
            archive_url="https://github.com/curl/curl/releases/download/curl-7_85_0/curl-7.85.0.tar.xz",
            hash="88b54a6d4b9a48cb4d873c7056dcba997ddd5b7be5a2d537a4acb55c20b04be6",
            dependencies=[
                "perl",
                "cmake",
                "ninja",
            ],
        )

    def build(self):
        CmakeProject.build(self, use_ninja=True)
        # Fix the pkg-config .pc file, correcting the library's names
        file_replace(
            os.path.join(self.pkg_dir, "lib", "pkgconfig", "libcurl.pc"),
            [
                (" -lcurl", " -llibcurl_imp"),
            ],
        )

        self.install(r".\COPYING share\doc\libcurl")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class OpenSSL(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "openssl",
            archive_url="https://www.openssl.org/source/openssl-1.1.1q.tar.gz",
            hash="d7939ce614029cdff0b6c20f0e2e5703158a489a72b2507b8bd51bf8c8fd10ca",
            dependencies=[
                "perl",
                "nasm",
                "msys2",
            ],
        )

    def build(self):
        common_options = r"no-ssl2 no-ssl3 no-comp --openssldir=%(gtk_dir)s/etc/ssl --prefix=%(gtk_dir)s"

        debug_option = "debug-" if self.builder.opts.configuration == "debug" else ""
        # Note that we want to give priority to the system perl version.
        # Using the msys2 one might endup giving us a broken build
        #        add_path = ';'.join([os.path.join(self.builder.perl_dir, 'bin'),
        #                             os.path.join(self.builder.opts.msys_dir, 'usr', 'bin')])
        add_path = None

        if self.builder.x86:
            self.exec_vs(
                r"%(perl_dir)s\bin\perl.exe Configure "
                + debug_option
                + "VC-WIN32 "
                + common_options
            )
        else:
            self.exec_vs(
                r"%(perl_dir)s\bin\perl.exe Configure "
                + debug_option
                + "VC-WIN64A "
                + common_options
            )

        try:
            self.exec_vs(r"nmake /nologo clean", add_path=add_path)
        except:  # noqa E722
            pass

        self.exec_vs(r"nmake /nologo", add_path=add_path)
        self.exec_vs(r"%(perl_dir)s\bin\perl.exe mk-ca-bundle.pl -n cert.pem")
        self.exec_vs(r"nmake /nologo install", add_path=add_path)

        self.install(r".\cert.pem bin")
        self.install(r".\LICENSE share\doc\openssl")
        self.install_pc_files()



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


"""Main build script."""

# Verify we can import from the script directory
try:
    import gvsbuild.utils.utils  # noqa: F401
except ImportError:
    # We are probably using an embedded installation
    print("Error importing utility, fixing paths ...")
    import os
    import sys

    # Get the script dir
    script_dir = os.path.dirname(os.path.realpath(sys.argv[0]))
    # and add it at the beginning, emulating the standard python startup
    sys.path.insert(0, script_dir)

import gvsbuild.groups  # noqa: F401
import gvsbuild.projects  # noqa: F401
import gvsbuild.tools  # noqa: F401
from gvsbuild.utils.parser import create_parser


def build():
    parser = create_parser()
    args = parser.parse_args()
    if hasattr(args, "func"):
        args.func(args)
    else:
        parser.print_help()



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GtkSourceView4(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gtksourceview4",
            archive_url="https://download.gnome.org/sources/gtksourceview/4.8/gtksourceview-4.8.3.tar.xz",
            hash="c30019506320ca2474d834cced1e2217ea533e00eb2a3f4eb7879007940ec682",
            dependencies=["python", "meson", "ninja", "gtk3", "pkg-config"],
        )
        if Project.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        else:
            self.add_param("-Dgir=false")
        self.add_param("-Dvapi=false")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gtksourceview4")


@project_add
class GtkSourceView5(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gtksourceview5",
            archive_url="https://download.gnome.org/sources/gtksourceview/5.6/gtksourceview-5.6.1.tar.xz",
            hash="659d9cc9d034a114f07e7e134ee80d77dec0497cb1516ae5369119c2fcb9da16",
            dependencies=["python", "meson", "ninja", "gtk4", "pkg-config"],
        )
        if Project.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        else:
            self.add_param("-Dintrospection=disabled")
        self.add_param("-Dvapi=false")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gtksourceview5")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gvsbuild', 'gvsbuild.projects', 'gvsbuild.utils']

package_data = \
{'': ['*'],
 'gvsbuild': ['patches/*',
              'patches/adwaita-icon-theme/Adwaita/64x64/ui/*',
              'patches/adwaita-icon-theme/Adwaita/96x96/ui/*',
              'patches/cairo/*',
              'patches/cairo/build/*',
              'patches/cairo/pc-files/*',
              'patches/cairo/util/cairo-gobject/*',
              'patches/check-libs/*',
              'patches/clutter/build/win32/vs12/*',
              'patches/clutter/build/win32/vs14/*',
              'patches/clutter/build/win32/vs15/*',
              'patches/cogl/*',
              'patches/cogl/GL/*',
              'patches/cogl/build/win32/vs12/*',
              'patches/cogl/build/win32/vs14/*',
              'patches/cogl/build/win32/vs15/*',
              'patches/cyrus-sasl/*',
              'patches/cyrus-sasl/include/*',
              'patches/cyrus-sasl/pc-files/*',
              'patches/emeus/*',
              'patches/enchant/src/*',
              'patches/ffmpeg/*',
              'patches/ffmpeg/build/*',
              'patches/freetype/*',
              'patches/gettext/*',
              'patches/gettext/msvc/gettext-runtime/gnulib-lib/*',
              'patches/gettext/msvc/gettext-runtime/gnulib-lib/sys/*',
              'patches/gettext/msvc/gettext-runtime/intl/*',
              'patches/gettext/msvc/gettext-runtime/libasprintf/*',
              'patches/gettext/msvc/gettext-tools/gnulib-lib/*',
              'patches/gettext/msvc/gettext-tools/gnulib-lib/libxml/*',
              'patches/gettext/msvc/gettext-tools/gnulib-lib/sys/*',
              'patches/gettext/msvc/gettext-tools/gnulib-tests/*',
              'patches/gettext/msvc/gettext-tools/gnulib-tests/arpa/*',
              'patches/gettext/msvc/gettext-tools/gnulib-tests/netinet/*',
              'patches/gettext/msvc/gettext-tools/gnulib-tests/sys/*',
              'patches/gettext/msvc/gettext-tools/libgettextpo/*',
              'patches/gettext/msvc/gettext-tools/libgettextpo/sys/*',
              'patches/gettext/msvc/gettext-tools/libgrep/*',
              'patches/gettext/msvc/gettext-tools/src/*',
              'patches/gettext/msvc/gettext-tools/src/textstyle/*',
              'patches/gettext/msvc/libtextstyle/lib/*',
              'patches/gettext/msvc/libtextstyle/lib/glib/*',
              'patches/gettext/msvc/libtextstyle/lib/libxml/*',
              'patches/gettext/msvc/libtextstyle/lib/sys/*',
              'patches/gettext/msvc/libtextstyle/lib/textstyle/*',
              'patches/gettext/msvc/vs12/Win32/gettext-runtime/*',
              'patches/gettext/msvc/vs12/Win32/gettext-runtime/gnulib-lib/*',
              'patches/gettext/msvc/vs12/Win32/gettext-tools/*',
              'patches/gettext/msvc/vs12/Win32/gettext-tools/gnulib-lib/*',
              'patches/gettext/msvc/vs12/Win32/gettext-tools/libgettextpo/*',
              'patches/gettext/msvc/vs12/Win32/libtextstyle/*',
              'patches/gettext/msvc/vs12/Win32/libtextstyle/lib/*',
              'patches/gettext/msvc/vs12/x64/gettext-runtime/*',
              'patches/gettext/msvc/vs12/x64/gettext-runtime/gnulib-lib/*',
              'patches/gettext/msvc/vs12/x64/gettext-tools/*',
              'patches/gettext/msvc/vs12/x64/gettext-tools/gnulib-lib/*',
              'patches/gettext/msvc/vs12/x64/gettext-tools/libgettextpo/*',
              'patches/gettext/msvc/vs12/x64/libtextstyle/*',
              'patches/gettext/msvc/vs12/x64/libtextstyle/lib/*',
              'patches/gettext/msvc/vs14/Win32/gettext-runtime/*',
              'patches/gettext/msvc/vs14/Win32/gettext-runtime/gnulib-lib/*',
              'patches/gettext/msvc/vs14/Win32/gettext-tools/*',
              'patches/gettext/msvc/vs14/Win32/gettext-tools/gnulib-lib/*',
              'patches/gettext/msvc/vs14/Win32/gettext-tools/libgettextpo/*',
              'patches/gettext/msvc/vs14/Win32/libtextstyle/*',
              'patches/gettext/msvc/vs14/Win32/libtextstyle/lib/*',
              'patches/gettext/msvc/vs14/arm64/gettext-runtime/*',
              'patches/gettext/msvc/vs14/arm64/gettext-runtime/gnulib-lib/*',
              'patches/gettext/msvc/vs14/arm64/gettext-tools/*',
              'patches/gettext/msvc/vs14/arm64/gettext-tools/gnulib-lib/*',
              'patches/gettext/msvc/vs14/arm64/gettext-tools/libgettextpo/*',
              'patches/gettext/msvc/vs14/arm64/libtextstyle/*',
              'patches/gettext/msvc/vs14/arm64/libtextstyle/lib/*',
              'patches/gettext/msvc/vs14/x64/gettext-runtime/*',
              'patches/gettext/msvc/vs14/x64/gettext-runtime/gnulib-lib/*',
              'patches/gettext/msvc/vs14/x64/gettext-tools/*',
              'patches/gettext/msvc/vs14/x64/gettext-tools/gnulib-lib/*',
              'patches/gettext/msvc/vs14/x64/gettext-tools/libgettextpo/*',
              'patches/gettext/msvc/vs14/x64/libtextstyle/*',
              'patches/gettext/msvc/vs14/x64/libtextstyle/lib/*',
              'patches/gettext/nmake/*',
              'patches/glib-networking/*',
              'patches/glib-py-wrapper/*',
              'patches/glib/*',
              'patches/gobject-introspection/*',
              'patches/gsettings-desktop-schemas/*',
              'patches/gst-plugins-bad/*',
              'patches/gtk2/*',
              'patches/gtk2/build/win32/*',
              'patches/gtk2/build/win32/vs12/*',
              'patches/gtk2/build/win32/vs14/*',
              'patches/gtk2/build/win32/vs15/*',
              'patches/gtk3/*',
              'patches/gtk3/gdk/win32/*',
              'patches/icu/pc-files-debug/*',
              'patches/icu/pc-files/*',
              'patches/leveldb/build/win32/vs12/*',
              'patches/leveldb/build/win32/vs12/leveldb_test/*',
              'patches/leveldb/build/win32/vs14/*',
              'patches/leveldb/build/win32/vs14/leveldb_test/*',
              'patches/leveldb/build/win32/vs15/*',
              'patches/leveldb/build/win32/vs15/leveldb_test/*',
              'patches/leveldb/port/*',
              'patches/leveldb/port/win/*',
              'patches/leveldb/util/*',
              'patches/libcroco/*',
              'patches/libcroco/win32/vs10/*',
              'patches/libcroco/win32/vs12/*',
              'patches/libcroco/win32/vs14/*',
              'patches/libcroco/win32/vs15/*',
              'patches/libffi/*',
              'patches/libmicrohttpd/*',
              'patches/libpng/*',
              'patches/libpng/pc-files/*',
              'patches/librsvg-legacy/*',
              'patches/librsvg-legacy/build/win32/vs12/*',
              'patches/librsvg-legacy/build/win32/vs14/*',
              'patches/librsvg-legacy/build/win32/vs15/*',
              'patches/libssh/*',
              'patches/libssh/build/vs12/*',
              'patches/libssh/build/vs14/*',
              'patches/libssh/build/vs15/*',
              'patches/libtiff-4/*',
              'patches/libvpx/*',
              'patches/libxml2/*',
              'patches/libxml2/include/libxml/*',
              'patches/libxml2/pc-files/*',
              'patches/libyuv/*',
              'patches/libyuv/pc-files/*',
              'patches/luajit/*',
              'patches/luajit/pc-files/*',
              'patches/lz4/pc-files/*',
              'patches/lz4/visual/VS2013/*',
              'patches/lz4/visual/VS2013/datagen/*',
              'patches/lz4/visual/VS2013/frametest/*',
              'patches/lz4/visual/VS2013/fullbench-dll/*',
              'patches/lz4/visual/VS2013/fullbench/*',
              'patches/lz4/visual/VS2013/fuzzer/*',
              'patches/lz4/visual/VS2013/liblz4-dll/*',
              'patches/lz4/visual/VS2013/liblz4/*',
              'patches/lz4/visual/VS2013/lz4/*',
              'patches/lz4/visual/VS2015/*',
              'patches/lz4/visual/VS2015/datagen/*',
              'patches/lz4/visual/VS2015/frametest/*',
              'patches/lz4/visual/VS2015/fullbench-dll/*',
              'patches/lz4/visual/VS2015/fullbench/*',
              'patches/lz4/visual/VS2015/fuzzer/*',
              'patches/lz4/visual/VS2015/liblz4-dll/*',
              'patches/lz4/visual/VS2015/liblz4/*',
              'patches/lz4/visual/VS2015/lz4/*',
              'patches/lz4/visual/VS2017/*',
              'patches/lz4/visual/VS2017/datagen/*',
              'patches/lz4/visual/VS2017/frametest/*',
              'patches/lz4/visual/VS2017/fullbench-dll/*',
              'patches/lz4/visual/VS2017/fullbench/*',
              'patches/lz4/visual/VS2017/fuzzer/*',
              'patches/lz4/visual/VS2017/liblz4-dll/*',
              'patches/lz4/visual/VS2017/liblz4/*',
              'patches/lz4/visual/VS2017/lz4/*',
              'patches/lz4/visual/VS2022/*',
              'patches/lz4/visual/VS2022/datagen/*',
              'patches/lz4/visual/VS2022/frametest/*',
              'patches/lz4/visual/VS2022/fullbench-dll/*',
              'patches/lz4/visual/VS2022/fullbench/*',
              'patches/lz4/visual/VS2022/fuzzer/*',
              'patches/lz4/visual/VS2022/liblz4-dll/*',
              'patches/lz4/visual/VS2022/liblz4/*',
              'patches/lz4/visual/VS2022/lz4/*',
              'patches/nghttp2/*',
              'patches/openssl/*',
              'patches/openssl/pc-files/*',
              'patches/pkgconf/*',
              'patches/protobuf-c/*',
              'patches/protobuf-c/pc-files/*',
              'patches/pygobject/*',
              'patches/win-iconv/*',
              'patches/x264/build/*',
              'patches/zlib/pc-files/*']}

extras_require = \
{'outdated': ['lastversion>=2.4.2,<3.0.0', 'packaging>=21.3,<22.0']}

entry_points = \
{'console_scripts': ['gvsbuild = gvsbuild.main:build']}

setup_kwargs = {
    'name': 'gvsbuild',
    'version': '2022.4.0',
    'description': 'GTK stack for Windows',
    'long_description': '# gvsbuild\n\n![CI](https://github.com/wingtk/gvsbuild/workflows/CI/badge.svg)\n\nThis python script helps you build a full [GTK](https://www.gtk.org/) library\nstack for Windows using Visual Studio. Currently, GTK 3 and GTK 4 are supported.\n\nThe script supports multiple versions of Visual Studio - at the moment we are\nfocusing on VS 2022, but we include projects for other versions, and we gladly\naccept patches.\n\nThe script focuses on GTK and the surrounding ecosystem (e.g. GStreamer).\nHowever, we are open to adding more libraries as long as the contributor takes\non the responsibility for keeping it up to date. The supported projects are\nmodules in the\n[projects](https://github.com/wingtk/gvsbuild/blob/master/gvsbuild/projects)\ndirectory.\n\nThe script requires a working installation of [Visual Studio for Windows\nDesktop](http://www.visualstudio.com), [Python 3](https://www.python.org) and\n[msys2](https://msys2.github.io). The script will download any additional tools\nrequired to build the libraries and will use them from a local directory,\nwithout any installation. As of today these tools include cmake, meson, ninja,\nnuget and perl.\n\nThe script fetches source tarballs for the projects from their original\nlocations, however in some cases it might be necessary to host a patched tarball\non GitHub. To ensure integrity of the downloaded files, the script checks the\nSHA256 hash of each download. Downloads are done using TLS, using SSL\ncertificates provided by the system, but in case of error the download is tried\nagain ignoring certificate errors.\n\n## Development Environment\n\n### Choco\nWe recommend using [Chocolately](https://chocolatey.org/) as a package manager\nin Windows.\n\nTo install it, open PowerShell as an administrator, then execute:\n\n```PowerShell\nSet-ExecutionPolicy Bypass -Scope Process -Force; iex ((New-Object System.Net.WebClient).DownloadString(\'https://community.chocolatey.org/install.ps1\'))\n```\nTo run local scripts in follow-on steps, also execute\n`Set-ExecutionPolicy RemoteSigned`. This allows for local PowerShell scripts\nto run without signing, but still requires signing for remote scripts.\n\n### Git\nTo setup a development environment in Windows install\n[Git](https://gitforwindows.org) by executing as an administrator:\n\n```PowerShell\nchoco install git\n```\n\n### MSYS2\nBoth of the development environments in the next steps need MSYS2 installed.\n\nInstall [MSYS2](http://www.msys2.org/):\n\nKeep PowerShell open as administrator and execute:\n```PowerShell\nchoco install msys2\n```\n\n### Building GTK\n\nFirst we will install the gvsbuild dependencies:\n1. Visual C++ build tools workload for Visual Studio 2022 Build Tools\n2. Python\n\n#### Install Visual Studio 2022\nWith your admin PowerShell terminal:\n\n```PowerShell\nchoco install visualstudio2022-workload-vctools\n```\n\nNote: Visual Studio versions 2013 (not for all projects), 2015, 2017, 2019, and 2022 are currently supported.\n\n#### Install the Latest Python\n\nDownload and install the latest version of Python:\n\n1. Install from Chocolately with `choco install python` with admin PowerShell\n1. Restart your PowerShell terminal as a normal user and check that `python --version` is correct.\n\nNote: If you are going to install Python using an alternative means, like the\nofficial Windows installers, we suggest to install Python in C:\\Python3x, for\nexample C:\\Python310. Alternatively you can use the `--python-dir` option to\ntell the script the correct location of your Python installation. Other Python\ndistributions like [Miniconda\n3](https://repo.continuum.io/miniconda/Miniconda3-latest-Windows-x86_64.exe)\nshould also work.\n\n#### Install gvsbuild\nThe recommended way to install gvsbuild is with pipx. Open a new regular user\nPowerShell terminal and execute:\n\n```PowerShell\npython -m pip install --user pipx\npython -m pipx ensurepath\npipx install gvsbuild\n```\n\nAlternatively, you can also use git to clone the repository and install it.\nOpen a new regular user PowerShell terminal and execute:\n\n```PowerShell\nmkdir C:\\gtk-build\\github\ncd C:\\gtk-build\\github\ngit clone https://github.com/wingtk/gvsbuild.git\ncd C:\\gtk-build\\github\\gvsbuild\npython -m venv .venv\n.\\.venv\\Scripts\\activate.ps1\npip install .\n```\n\n#### Build GTK\n\nIn the same PowerShell terminal, execute:\n\n```PowerShell\ngvsbuild build gtk3\n```\n\nAlternatively, if you want to build GTK 4, execute:\n```PowerShell\ngvsbuild build gtk4\n```\n\nGrab a coffee, the build will take a few minutes to complete.\n\n#### Add GTK to Your Path\n\n```PowerShell\n$env:Path = "C:\\gtk-build\\gtk\\x64\\release\\bin;" + $env:Path\n```\n\n#### Other Options\n\n For more information about the possible commands run:\n\n ```\n gvsbuild --help\n ```\n\n To get detailed help on the build command run:\n\n ```\n gvsbuild build --help\n ```\n\n It is possible to set some parameters from a file, e.g. vs2015-release.pro, putting\n the @ character before the file name. The file contains the option, one per\n line, separated by a carriage return:\n\n ```\n --vs-ver\n 14\n --win-sdk\n 8.1\n -c\n release\n ```\n\n Even if the format is not the easier to write or read in this way we eliminate\n the problem of escaping spaces is file names and directories. Then you can use\n it:\n\n ```\n gvsbuild build @vs2015-release.pro gtk3-full\n ```\n\n## Troubleshooting\n\nIf the download of a tarball fails a partial file will not pass the hash check,\ndelete the file and try again.\n\n## OpenSSL\n\nIn addition to the setup instructions above, to build OpenSSL you also need the\nVisual C++ 2013 Redistributable Package installed. To install it, open PowerShell\nas administrator and execute:\n\n```PowerShell\nchoco install vcredist2013\n```\n\nSimilar to other packages, you can build OpenSSL by executing:\n```\ngvsbuild build openssl\n```\n\n## Dependency Graph\n\nTo see and analyze the dependency between the various projects, in text or in a\nGraphviz format, use the script deps.py:\n\n ```\n cd C:\\gtk-build\\github\\gvsbuild\n python .\\deps.py -g -o test.gv\n ```\n\nWithout option a simple dependency of all the projects is printed, as usual with\n--help a summary of the options/commands is printed.\n\n## License\n\nThis build script is licensed under the GPL2.0 license, see the COPYING file for\nthe full text.\n\nThe binaries produced by the build script are licensed under the license terms\nof the project that is built (ie glib is LGPL so you can use glib.dll built\nwith this script under the terms of LGPL).\n\nPatches included in the repository are licensed under the license terms of the\nproject they apply to.\n\n## Credits\n\nThis tool originated from a gtk-win32 PowerShell script created by the\n[HexChat](https://hexchat.github.io/) developers for building it for Windows.\nAlthough this project is now archived, you can explore the original project if you\nare interested in the history at https://github.com/hexchat/gtk-win32.\n\nCompiling the GTK stack on MSVC would not be possible without the incredible\nwork by [Fan Chun-wei](https://github.com/fanc999). If you are interested in more\ndetails of how this works, please see [Compiling the GTK+ (and Clutter) stack using\nVisual C++ 2008 and\nlater](https://wiki.gnome.org/Projects/GTK/Win32/MSVCCompilationOfGTKStack).\n',
    'author': 'Ignacio Casal Quinteiro',
    'author_email': 'qignacio@amazon.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class JsonGLib(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "json-glib",
            archive_url="https://download.gnome.org/sources/json-glib/1.6/json-glib-1.6.6.tar.xz",
            hash="96ec98be7a91f6dde33636720e3da2ff6ecbb90e76ccaa49497f31a6855a490e",
            dependencies=["meson", "ninja", "pkg-config", "python", "glib"],
        )
        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
            enable_gi = "enabled"
        else:
            enable_gi = "disabled"

        self.add_param("-Dgtk_doc=disabled")
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self, make_tests=True)

        self.install(r".\COPYING share\doc\json-glib")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add
from gvsbuild.utils.utils import convert_to_msys


@project_add
class Ffmpeg(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "ffmpeg",
            archive_url="https://ffmpeg.org/releases/ffmpeg-5.1.2.tar.xz",
            hash="619e706d662c8420859832ddc259cd4d4096a48a2ce1eefd052db9e440eef3dc",
            dependencies=["nasm", "msys2", "pkg-config", "nv-codec-headers"],
            patches=["0001-Added-support-for-MB_INFO.patch"],
        )
        if self.opts.ffmpeg_enable_gpl:
            self.add_dependency("x264")

    def build(self):
        msys_path = Project.get_tool_path("msys2")
        self.exec_vs(
            r"%s\bash build\build.sh %s %s %s %s"
            % (
                msys_path,
                convert_to_msys(self.pkg_dir),
                convert_to_msys(self.builder.gtk_dir),
                self.builder.opts.configuration,
                "enable_gpl" if self.opts.ffmpeg_enable_gpl else "disable_gpl",
            ),
            add_path=msys_path,
        )

        self.install(r".\COPYING.LGPLv2.1 " r".\COPYING.LGPLv3 " r"share\doc\ffmpeg")
        if self.opts.ffmpeg_enable_gpl:
            self.install(r".\COPYING.GPLv2 " r"share\doc\ffmpeg")

    def post_install(self):
        self.builder.exec_msys(
            ["mv", "avcodec.lib", "../lib/"],
            working_dir=os.path.join(self.builder.gtk_dir, "bin"),
        )
        self.builder.exec_msys(
            ["mv", "avutil.lib", "../lib/"],
            working_dir=os.path.join(self.builder.gtk_dir, "bin"),
        )
        self.builder.exec_msys(
            ["mv", "swscale.lib", "../lib/"],
            working_dir=os.path.join(self.builder.gtk_dir, "bin"),
        )


@project_add
class Project_nv_codec_headers(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "nv-codec-headers",
            archive_url="https://github.com/FFmpeg/nv-codec-headers/releases/download/n11.1.5.1/nv-codec-headers-11.1.5.1.tar.gz",
            hash="a28cdde3ac0e9e02c2dde7a1b4de5333b4ac6148a8332ca712da243a3361a0d9",
            version="11.1.5.1",
        )

    def build(self):
        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        self.exec_vs(r'make install PREFIX="%(gtk_dir)s"', add_path=add_path)



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Cairo(Tarball, Project):
    def __init__(self):
        Project.__init__(
            self,
            "cairo",
            archive_url="https://cairographics.org/releases/cairo-1.16.0.tar.xz",
            hash="5e7b29b3f113ef870d1e3ecf8adf21f923396401604bda16d44be45e66052331",
            dependencies=["fontconfig", "glib", "pixman", "libpng"],
            patches=["0001-Fix-mask-usage-in-image-compositor.patch"],
        )

    def build(self):
        self.exec_vs(
            r"make -f Makefile.win32 CFG=%(configuration)s ARCH=%(platform)s",
            add_path=os.path.join(self.builder.opts.msys_dir, "usr", "bin"),
        )
        self.push_location(r".\util\cairo-gobject")
        self.exec_vs(
            r"make -f Makefile.win32 CFG=%(configuration)s ARCH=%(platform)s",
            add_path=os.path.join(self.builder.opts.msys_dir, "usr", "bin"),
        )
        self.pop_location()

        self.install(r".\src\%(configuration)s\cairo.dll bin")
        self.install(r".\util\cairo-gobject\%(configuration)s\cairo-gobject.dll bin")

        self.install(r".\src\%(configuration)s\cairo.lib lib")
        self.install(r".\util\cairo-gobject\%(configuration)s\cairo-gobject.lib lib")

        self.install(r".\src\cairo.h include\cairo")
        self.install(r".\src\cairo-deprecated.h include\cairo")
        self.install(r".\src\cairo-pdf.h include\cairo")
        self.install(r".\src\cairo-ps.h include\cairo")
        self.install(r".\src\cairo-script.h include\cairo")
        self.install(r".\src\cairo-svg.h include\cairo")
        self.install(r".\src\cairo-tee.h include\cairo")
        self.install(r".\src\cairo-win32.h include\cairo")
        self.install(r".\src\cairo-xml.h include\cairo")
        self.install(r".\src\cairo-ft.h include\cairo")
        self.install(r".\src\cairo-features.h include\cairo")
        self.install(r".\util\cairo-gobject\cairo-gobject.h include\cairo")
        self.install(r".\cairo-version.h include\cairo")

        self.install_pc_files()
        self.install(r".\COPYING share\doc\cairo")



#  Copyright (C) 2016 - Yevgen Muntyan
#  Copyright (C) 2016 - Ignacio Casal Quinteiro
#  Copyright (C) 2016 - Arnavion
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

"""Base tool class, from the project one."""

import os

from .base_project import Project, ProjectType


class Tool(Project):
    def __init__(self, name, **kwargs):
        self.dir_part = None
        self.mark_deps = False
        self.tool_path = None
        self.full_exe = None
        Project.__init__(self, name, **kwargs)

    def load_defaults(self):
        if self.dir_part:
            self.build_dir = os.path.join(self.opts.tools_root_dir, self.dir_part)
        else:
            self.build_dir = os.path.join(self.opts.tools_root_dir, self.name)
        if hasattr(self, "exe_name"):
            self.full_exe = os.path.join(self.build_dir, self.exe_name)

    def tool_mark(self):
        # Create the directory to let the --fast-build option work as expected
        if not os.path.exists(self.build_dir):
            os.makedirs(self.build_dir)
            self.mark_deps = True

    def build(self):
        # All the work is done in the unpack & we don't force the rebuild of all projects that uses this tool
        return not self.mark_deps

    def update_build_dir(self):
        self.unpack()

    def get_path(self):
        return self.tool_path or self.build_dir

    def get_executable(self):
        if self.full_exe:
            return self.full_exe
        raise NotImplementedError(f"{self.name}:get_executable")

    def get_base_dir(self):
        """Base directory for the tool, used for perl to have the dir to pass
        to the *make tool, normally not used (we update the path or use
        directly the executable)"""
        raise NotImplementedError("get_base_dir")

    def export(self):
        # We do not want to export tools
        pass


def tool_add(cls):
    """Class decorator to add the newly created Toolp class to the global
    projects/tools/groups list."""
    Project.register(cls, ProjectType.TOOL)
    return cls
